@@ -68,21 +68,28 @@ class MainWindow(QtWidgets.QMainWindow):
         old_cwd = os.getcwd()
         ui_dir = os.path.dirname(__file__)
         os.chdir(ui_dir)
+        # Test to see if we're launching from a git branch, if so the title
+        # bar will be updated for easy reference.
+
+        # Used to hide the stderr from the user as it doesn't matter
+        f = open(nxt_io.generate_temp_file('NxtGitErr'))
         try:
-            git_out = subprocess.check_output(["git", "branch"]).decode("utf8")
+            git_out = subprocess.check_output(["git", "branch"],
+                                              stderr=f).decode("utf8")
             cur = next(line for line in git_out.split("\n")
                        if line.startswith("*"))
             current_branch = cur.strip("*").strip()
         except:  # Broad because Maya
-            # logger.exception("Failed to run git branch, attempting fallback "
-            #                  "method.")
+            # Failed to run git branch, attempting fallback method
             try:
                 with open('../../.git/HEAD') as f:
                     head = f.read()
                 _, __, current_branch = head.rpartition('/')
-            except IOError:
-                # logger.exception("Could not determine git branch.")
+            except:
+                # Could not determine git branch, must be pip package.
                 current_branch = ''
+        finally:
+            f.close()
         os.chdir(old_cwd)
         if is_standalone():
             context = 'standalone'
@@ -97,6 +104,9 @@ class MainWindow(QtWidgets.QMainWindow):
                                                     '.'.join([str(n) for n in sys.version_info[:3]]),
                                                     current_branch))
         self.setObjectName('Main Window')
+        self.zoom_keys = QtGui.QKeySequence(QtCore.Qt.Key_Alt)
+        self.zoom_keys_down = False
+        self._held_keys = []
         self._closing = False
         self.last_focused_start = 0  # Start point focus tracker
         # FIXME: Fix with MV signal
@@ -737,8 +747,24 @@ class MainWindow(QtWidgets.QMainWindow):
         self.property_editor.update_styles()
 
     def keyPressEvent(self, event):
+        key = event.key()
+        if key not in self._held_keys:
+            self._held_keys.append(key)
+        self.zoom_keys_down = False
+        match = QtGui.QKeySequence(*self._held_keys).matches(self.zoom_keys)
+        if match == QtGui.QKeySequence.SequenceMatch.ExactMatch:
+            self.zoom_keys_down = True
         event.accept()
 
+    def keyReleaseEvent(self, event):
+        key = event.key()
+        if key in self._held_keys:
+            self._held_keys.remove(key)
+        self.zoom_keys_down = False
+        match = QtGui.QKeySequence(*self._held_keys).matches(self.zoom_keys)
+        if match == QtGui.QKeySequence.SequenceMatch.ExactMatch:
+            self.zoom_keys_down = True
+
     def eventFilter(self, widget, event):
         # enter editing after update_code_is_local
         if event.type() == QtCore.QEvent.MouseButtonDblClick:
